@@ -1,7 +1,7 @@
 /*
  * parameters.h - parameter init/load/save code - header file
  *
- * Copyright (c) 2001-2006 ARAnyM developer team (see AUTHORS)
+ * Copyright (c) 2001-2008 ARAnyM developer team (see AUTHORS)
  *
  * Authors:
  *  MJ		Milan Jurik
@@ -29,8 +29,6 @@
 #define PARAMETERS_H
 
 #include "sysdeps.h"
-#include "version.h"
-#include "cfgopts.h"
 #include <SDL_keyboard.h>
 
 #ifndef ARANYMHOME
@@ -59,7 +57,7 @@ enum geo_type {
 	geoByteswap
 };
 
-extern int get_geometry(char *, enum geo_type geo);
+extern int get_geometry(const char *, enum geo_type geo);
 
 // External filesystem type
 typedef struct {
@@ -105,6 +103,7 @@ typedef struct {
 typedef struct {
   bool redirect_CON;
   long cookie_mch;
+  long cookie_akp;
 } bx_tos_options_t;
 
 // Video output options
@@ -227,6 +226,24 @@ typedef struct {
 	SDL_keysym	screenshot;
 } bx_hotkeys_t;
 
+// Audio
+typedef struct {
+	uint32 freq;
+	uint32 chans;
+	uint32 bits;
+	uint32 samples;
+} bx_audio_options_t;
+
+// Joysticks
+typedef struct {
+	int32	ikbd0;
+	int32	ikbd1;
+	int32	joypada;
+	char	joypada_mapping[256];
+	int32	joypadb;
+	char	joypadb_mapping[256];
+} bx_joysticks_options_t;
+
 #define DISKS	8
 
 // Options 
@@ -253,6 +270,8 @@ typedef struct {
   bx_parallel_options_t parallel;
   bx_natfeat_options_t natfeats;
   bx_nfvdi_options_t	nfvdi;
+  bx_audio_options_t	audio;
+  bx_joysticks_options_t	joysticks;
   char			tos_path[512];
   char			emutos_path[512];
   char			bootstrap_path[512];
@@ -276,10 +295,11 @@ extern char emutos_path[512];
 extern bool startupGUI;
 extern bool boot_emutos;
 extern bool boot_lilo;
+extern bool halt_on_reboot;
 
 void usage(int);
 extern bool check_cfg();
-extern bool decode_switches(FILE *, int, char **);
+extern bool decode_switches(int, char **);
 extern char *getConfFilename(const char *file, char *buffer, unsigned int bufsize);
 extern char *getDataFilename(const char *file, char *buffer, unsigned int bufsize);
 char *addFilename(char *buffer, const char *file, unsigned int bufsize);
