@@ -19,15 +19,22 @@
 #ifndef VERSION_H
 #define VERSION_H
 
+#include "version_date.h"
+#ifdef VERSION_DATE
+#define CVS_DATE	"CVS " VERSION_DATE
+#else
+#define CVS_DATE	"CVS"
+#endif
+
 #define VER_MAJOR	0
 #define VER_MINOR	9
-#define VER_MICRO	5
+#define VER_MICRO	7
 
 #ifndef VER_STATUS
 //#define VER_STATUS
 //#define VER_STATUS	"alpha"
 #define VER_STATUS	"beta"
-//#define VER_STATUS	"beta+cvs"
+//#define VER_STATUS	"beta+" CVS_DATE
 #endif
 
 #define str(x)		_stringify (x)
