@@ -1,7 +1,7 @@
 /*
  * input.cpp - handling of keyboard/mouse input
  *
- * Copyright (c) 2001-2006 Petr Stehlik of ARAnyM dev team (see AUTHORS)
+ * Copyright (c) 2001-2008 Petr Stehlik of ARAnyM dev team (see AUTHORS)
  *
  * This file is part of the ARAnyM project which builds a new and powerful
  * TOS/FreeMiNT compatible virtual machine running on almost any hardware.
@@ -29,7 +29,8 @@
 #include "main.h"			// for RestartAll()
 #include "ata.h"
 #ifdef SDL_GUI
-#  include "sdlgui.h"
+# include "gui-sdl/sdlgui.h"
+# include "gui-sdl/dialog.h"
 #endif
 
 #ifdef NFVDI_SUPPORT
@@ -42,6 +43,31 @@
 
 #include <SDL.h>
 
+/* Joysticks */
+
+enum {
+	ARANYM_JOY_IKBD0=0,
+	ARANYM_JOY_IKBD1,
+	ARANYM_JOY_JOYPADA,
+	ARANYM_JOY_JOYPADB
+};
+
+#define OPEN_JOYSTICK(host_number, array_number) \
+	if (host_number>=0) {	\
+		sdl_joysticks[array_number] = \
+			SDL_JoystickOpen(host_number);	\
+		if (!sdl_joysticks[array_number]) {	\
+			panicbug("Could not open joystick %d",	\
+				host_number);	\
+		}	\
+	}
+
+SDL_Joystick *sdl_joysticks[4]={
+	NULL, NULL, NULL, NULL
+};
+
+
+
 /*
  * Four different types of keyboard translation:
  *
@@ -88,14 +114,22 @@ static void hideMouse(bool hide)
 
 void InputInit()
 {
-	// warp mouse to center of Atari 320x200 screen and grab it
-	if (! bx_options.video.fullscreen)
-		SDL_WarpMouse(320/2, 200/2);
-	grabMouse(true);
-	// hide mouse unconditionally
-	hideMouse(true);
+	if (bx_options.startup.grabMouseAllowed) {
+		// warp mouse to center of Atari 320x200 screen and grab it
+		if (! bx_options.video.fullscreen)
+			SDL_WarpMouse(320/2, 200/2);
+		grabMouse(true);
+		// hide mouse unconditionally
+		hideMouse(true);
+	}
 	// capslockState (yes, 'false' is correct)
 	capslockState = false;
+
+	/* Open joysticks */
+	OPEN_JOYSTICK(bx_options.joysticks.ikbd0, ARANYM_JOY_IKBD0);
+	OPEN_JOYSTICK(bx_options.joysticks.ikbd1, ARANYM_JOY_IKBD1);
+	OPEN_JOYSTICK(bx_options.joysticks.joypada, ARANYM_JOY_JOYPADA);
+	OPEN_JOYSTICK(bx_options.joysticks.joypadb, ARANYM_JOY_JOYPADB);
 }
 
 void InputReset()
@@ -109,12 +143,20 @@ void InputExit()
 {
 	grabMouse(false);	// release mouse
 	hideMouse(false);	// show it
+
+	/* Close joysticks */
+	int i;
+	for (i=0; i<4; i++) {
+		if (sdl_joysticks[i]) {
+			SDL_JoystickClose(sdl_joysticks[i]);
+		}
+	}
 }
 
 bool grabMouse(bool grab)
 {
 	int current = SDL_WM_GrabInput(SDL_GRAB_QUERY);
-	if (bx_options.startup.grabMouseAllowed && grab && current != SDL_GRAB_ON) {
+	if (grab && current != SDL_GRAB_ON) {
 		SDL_WM_GrabInput(SDL_GRAB_ON);
 		grabbedMouse = true;
 		hideMouse(true);
@@ -185,16 +227,19 @@ static int keyboardTable[0x80] = {
 
 static int keysymToAtari(SDL_keysym keysym)
 {
- 
-//fprintf (stdout, "scancode: %x - sym: %x - char: %s\n", keysym.scancode, keysym.sym, SDL_GetKeyName (keysym.sym));
+    // panicbug("scancode: %x - sym: %x - char: %s", keysym.scancode, keysym.sym, SDL_GetKeyName (keysym.sym));
 
 	int sym = keysym.scancode;
 
 	switch (keysym.sym) {
 	  case SDLK_LMETA:
 	  case SDLK_RMETA:
-	    return 0;
-		break;
+		#if MAP_META_TO_CONTROL
+		  return 0x1D;
+		#else		
+		  return 0;
+		#endif
+		  break;
 	  case SDLK_MODE: /* passthru */ /* Alt Gr key according to SDL docs */
 	  case SDLK_RALT:
 		return RALT_ATARI_SCANCODE;
@@ -209,13 +254,17 @@ static int keysymToAtari(SDL_keysym keysym)
 		break;
 	  case SDLK_RCTRL:
 	  case SDLK_LCTRL:
-	    return 0x1D;
+		#if MAP_CONTROL_TO_CONTROL
+		  return 0x1D;
+		#else		
+		  return 0;
+		#endif
 	  default:
 	    break;
 	}
 	for (int i = 0; i < 0x73; i++) {
 		if (keyboardTable[i] == sym) {
-//		fprintf (stdout, "scancode mac:%x - scancode atari: %x\n", keysym.scancode, i);
+//			panicbug ("scancode mac:%x - scancode atari: %x", keysym.scancode, i);
 			return i;
 		}
 	}
@@ -374,7 +423,7 @@ static int keysymToAtari(SDL_keysym keysym)
 		case SDLK_F11:	return 0x62;	/* F11 => Help */
 		case SDLK_F12:	return 0x61;	/* F12 => Undo */
 		case SDLK_HOME:	return 0x47;	/* Home */
-		case SDLK_END:	return 0x60;	/* End => "<>" on German Atari kbd */
+		case SDLK_END:	return 0x4f;	/* Milan's scancode for End */
 		case SDLK_UP:	return 0x48;	/* Arrow Up */
 		case SDLK_LEFT:	return 0x4b;	/* Arrow Left */
 		case SDLK_RIGHT:	return 0x4d;	/* Arrow Right */
@@ -382,6 +431,9 @@ static int keysymToAtari(SDL_keysym keysym)
 		case SDLK_INSERT:	return 0x52;	/* Insert */
 		case SDLK_DELETE:	return 0x53;	/* Delete */
 
+		// a key that is out of the fixed offset matrix
+		case SDLK_LESS: return 0x60;	/* a '<>' key next to short left Shift */
+
 		// keys not found on original Atari keyboard
 		case SDLK_RCTRL:	return 0x1d;	/* map right Control to Atari control */
 		case SDLK_MODE: /* passthru */ /* Alt Gr key according to SDL docs */
@@ -421,57 +473,51 @@ static bool mouseOut = false;
 
 #ifdef SDL_GUI
 extern bool isGuiAvailable;	// from main.cpp
-static SDL_Thread *GUIthread = NULL;
-static const int GUI_RETURN_INFO = (SDL_USEREVENT+1);
+static bool cur_fullscreen;
 
-// running in a different thread
-static int open_gui(void * /*ptr*/)
+void open_GUI(void)
 {
-	bool fullscreen = bx_options.video.fullscreen;
+	if (!isGuiAvailable || !SDLGui_isClosed()) {
+		return;
+	}
 
-	hostScreen.openGUI();
-	int status = GUImainDlg();
+	cur_fullscreen = bx_options.video.fullscreen;
 
-	// the status is sent to event checking thread by the USEREVENT+1 message
-	SDL_Event ev;
-	ev.type = GUI_RETURN_INFO;
-	ev.user.code = status;	// STATUS_SHUTDOWN or STATUS_REBOOT
-	ev.user.data1 = NULL;
-	SDL_PeepEvents(&ev, 1, SDL_ADDEVENT, SDL_EVENTMASK(GUI_RETURN_INFO));
+	/* Always ungrab+show mouse */
+	SDL_ShowCursor(SDL_ENABLE);
+	SDL_WM_GrabInput(SDL_GRAB_OFF);
+	
+	SDLGui_Open(NULL);
+}
 
-	hostScreen.closeGUI();
+void close_GUI(void)
+{
+	if (!isGuiAvailable /*|| SDLGui_isClosed()*/) {
+		return;
+	}
+
+	SDLGui_Close();
 
 	// small hack to toggle fullscreen from the SETUP GUI
-	if (bx_options.video.fullscreen != fullscreen) {
-		bx_options.video.fullscreen = fullscreen;
+	if (bx_options.video.fullscreen != cur_fullscreen) {
+		bx_options.video.fullscreen = cur_fullscreen;
 
-		hostScreen.toggleFullScreen();
+		host->video->toggleFullScreen();
 		if (bx_options.video.fullscreen && !grabbedMouse)
 			grabTheMouse();
 	}
 
-	return 0;
+	/* Restore mouse cursor state */
+	if (hiddenMouse) {
+		SDL_ShowCursor(SDL_DISABLE);
+	}
+	if (grabbedMouse) {
+		SDL_WM_GrabInput(SDL_GRAB_ON);
+	}
+
+	host->video->forceRefreshScreen();
 }
 
-bool start_GUI_thread()
-{
-	if (isGuiAvailable && !hostScreen.isGUIopen()
-#ifdef ENABLE_OPENGL
-		&& !bx_options.opengl.enabled
-#endif
-	) {
-		GUIthread = SDL_CreateThread(open_gui, NULL);
-	}
-	return (GUIthread != NULL);
-}
-
-void kill_GUI_thread()
-{
-	if (GUIthread != NULL) {
-		SDL_KillThread(GUIthread);
-		GUIthread = NULL;
-	}
-}
 #endif /* SDL_GUI */
 
 #define CHECK_HOTKEY(Hotkey) ((bx_options.hotkeys.Hotkey.sym == 0 || sym == bx_options.hotkeys.Hotkey.sym) && masked_mod == bx_options.hotkeys.Hotkey.mod)
@@ -479,16 +525,30 @@ static void process_keyboard_event(const SDL_Event &event)
 {
 	SDL_keysym keysym = event.key.keysym;
 	SDLKey sym = keysym.sym;
-	int state = SDL_GetModState(); // keysym.mod does not deliver single mod key presses for some reason
-
+	int state;
+	
+	if ((keysym.mod & HOTKEYS_MOD_MASK) == 0)
+		state  = SDL_GetModState(); // keysym.mod does not deliver single mod key presses for some reason
+	else
+		state = keysym.mod;	// May be send by SDL_PushEvent
+		
 #ifdef SDL_GUI
-	if (hostScreen.isGUIopen()) {
-		SDL_Event ev;
-		ev.type = SDL_USEREVENT;	// map key down/up event to user event
-		ev.user.code = event.type;
-		ev.user.data1 = (void *)(uintptr)sym;
-		ev.user.data2 = (void *)(uintptr)state;
-		SDL_PeepEvents(&ev, 1, SDL_ADDEVENT, SDL_EVENTMASK(SDL_USEREVENT));
+	if (!SDLGui_isClosed()) {
+		switch (SDLGui_DoEvent(event)) {
+			case Dialog::GUI_CLOSE:
+				close_GUI();
+				break;
+			case Dialog::GUI_REBOOT:
+				close_GUI();
+				RestartAll();
+				break;
+			case Dialog::GUI_SHUTDOWN:
+				close_GUI();
+				pendingQuit = true;
+				break;
+			default:
+				break;
+		}
 		return;	// don't pass the key events to emulation
 	}
 #endif /* SDL_GUI */
@@ -546,10 +606,10 @@ static void process_keyboard_event(const SDL_Event &event)
 				getIKBD()->SendKey(0x80 | 0x1d);
 			if ( bx_options.hotkeys.setup.mod & KMOD_LALT )
 				getIKBD()->SendKey(0x80 | 0x38);
-			if ( bx_options.hotkeys.setup.mod & KMOD_MODE|KMOD_RALT )
+			if ( bx_options.hotkeys.setup.mod & (KMOD_MODE|KMOD_RALT) )
 				getIKBD()->SendKey(0x80 | RALT_ATARI_SCANCODE);
 			
-			start_GUI_thread();
+			open_GUI();
 			send2Atari = false;
 		}
 #endif
@@ -564,17 +624,17 @@ static void process_keyboard_event(const SDL_Event &event)
 #endif
 		else if (CHECK_HOTKEY(ungrab)) {
 			if ( bx_options.video.fullscreen )
-				hostScreen.toggleFullScreen();
+				host->video->toggleFullScreen();
 			releaseTheMouse();
 			canGrabMouseAgain = false;	// let it leave our window
 			send2Atari = false;
 		}
 		else if (CHECK_HOTKEY(screenshot)) {
-			hostScreen.makeSnapshot();
+			host->video->doScreenshot();
 			send2Atari = false;
 		}
 		else if (CHECK_HOTKEY(fullscreen)) {
-			hostScreen.toggleFullScreen();
+			host->video->toggleFullScreen();
 			if (bx_options.video.fullscreen && !grabbedMouse)
 				grabTheMouse();
 			send2Atari = false;
@@ -635,15 +695,21 @@ static void process_mouse_event(const SDL_Event &event)
 #endif
 
 #ifdef SDL_GUI
-	if (hostScreen.isGUIopen()) {
-		int typ = event.type;
-		if (typ == SDL_MOUSEBUTTONDOWN || typ == SDL_MOUSEBUTTONUP) {
-			SDL_Event ev;
-			ev.type = SDL_USEREVENT;	// map button down/up to user event
-			ev.user.code = typ;
-			ev.user.data1 = (void *)(uintptr)event.button.x;
-			ev.user.data2 = (void *)(uintptr)event.button.y;
-			SDL_PeepEvents(&ev, 1, SDL_ADDEVENT, SDL_EVENTMASK(SDL_USEREVENT));
+	if (!SDLGui_isClosed()) {
+		switch (SDLGui_DoEvent(event)) {
+			case Dialog::GUI_CLOSE:
+				close_GUI();
+				break;
+			case Dialog::GUI_REBOOT:
+				close_GUI();
+				RestartAll();
+				break;
+			case Dialog::GUI_SHUTDOWN:
+				close_GUI();
+				pendingQuit = true;
+				break;
+			default:
+				break;
 		}
 		return;	// don't pass the mouse events to emulation
 	}
@@ -755,7 +821,7 @@ static void process_mouse_event(const SDL_Event &event)
 #endif
 
 	// send the mouse data packet
-	if (xrel || yrel || lastbut != but) {
+	if ((xrel || yrel || lastbut != but) && grabbedMouse) {
 		if (xrel < -250 || xrel > 250 || yrel < -250 || yrel > 250) {
 			bug("Reseting weird mouse packet: %d, %d, %d", xrel, yrel, but);
 			xrel = yrel = 0;	// reset the values otherwise ikbd gets crazy
@@ -770,8 +836,11 @@ static void process_mouse_event(const SDL_Event &event)
 			mouseOut = true;
 
 		// same check but for bottom and right side of our window
-		if ((xrel > 0 && getARADATA()->getAtariMouseX() >= (int32)hostScreen.getWidth() - 1) ||
-			(yrel > 0 && getARADATA()->getAtariMouseY() >= (int32)hostScreen.getHeight() - 1))
+		int hostw = host->video->getWidth();
+		int hosth = host->video->getHeight();
+
+		if ((xrel > 0 && getARADATA()->getAtariMouseX() >= (int32) hostw - 1) ||
+			(yrel > 0 && getARADATA()->getAtariMouseY() >= (int32) hosth - 1))
 			mouseOut = true;
 	}
 }
@@ -783,7 +852,13 @@ static void process_active_event(const SDL_Event &event)
 
 		// if it's mouse focus event
 		if (event.active.state == SDL_APPMOUSEFOCUS) {
-
+#ifdef SDL_GUI
+			// Disable grab, if Setup GUI is open
+			// because else the mouse goes invisible if the window is reentered
+			if (!SDLGui_isClosed()) {
+				return;
+			}
+#endif
 			// if we can grab the mouse automatically
 			// and if the Atari mouse driver works
 			if (bx_options.autoMouseGrab && getARADATA()->isAtariMouseDriver()) {
@@ -817,24 +892,67 @@ static void process_active_event(const SDL_Event &event)
 
 /*--- Joystick event ---*/
 
-SDL_Joystick *sdl_joystick;
-
 static void process_joystick_event(const SDL_Event &event)
 {
 	switch(event.type) {
 		case SDL_JOYAXISMOTION:
-			getIKBD()->SendJoystickAxis(1, event.jaxis.axis, event.jaxis.value);
+			if (event.jaxis.which==bx_options.joysticks.ikbd0) {
+				getIKBD()->SendJoystickAxis(0, event.jaxis.axis, event.jaxis.value);
+			} else if (event.jaxis.which==bx_options.joysticks.ikbd1) {
+				getIKBD()->SendJoystickAxis(1, event.jaxis.axis, event.jaxis.value);
+			} else if (event.jaxis.which==bx_options.joysticks.joypada) {
+				getJOYPADS()->sendJoystickAxis(0, event.jaxis.axis, event.jaxis.value);
+			} else if (event.jaxis.which==bx_options.joysticks.joypadb) {
+				getJOYPADS()->sendJoystickAxis(1, event.jaxis.axis, event.jaxis.value);
+			}
 			break;		
 		case SDL_JOYHATMOTION:
-			getIKBD()->SendJoystickHat(1, event.jhat.value);
+			if (event.jaxis.which==bx_options.joysticks.ikbd0) {
+				getIKBD()->SendJoystickHat(0, event.jhat.value);
+			} else if (event.jaxis.which==bx_options.joysticks.ikbd1) {
+				getIKBD()->SendJoystickHat(1, event.jhat.value);
+			} else if (event.jaxis.which==bx_options.joysticks.joypada) {
+				getJOYPADS()->sendJoystickHat(0, event.jhat.value);
+			} else if (event.jaxis.which==bx_options.joysticks.joypadb) {
+				getJOYPADS()->sendJoystickHat(1, event.jhat.value);
+			}
 			break;		
 		case SDL_JOYBUTTONDOWN:
 		case SDL_JOYBUTTONUP:
-			getIKBD()->SendJoystickButton(1, event.jbutton.state==SDL_PRESSED);
+			if (event.jaxis.which==bx_options.joysticks.ikbd0) {
+				getIKBD()->SendJoystickButton(0,
+					event.jbutton.state==SDL_PRESSED);
+			} else if (event.jaxis.which==bx_options.joysticks.ikbd1) {
+				getIKBD()->SendJoystickButton(1,
+					event.jbutton.state==SDL_PRESSED);
+			} else if (event.jaxis.which==bx_options.joysticks.joypada) {
+				getJOYPADS()->sendJoystickButton(0, event.jbutton.button,
+					event.jbutton.state==SDL_PRESSED);
+			} else if (event.jaxis.which==bx_options.joysticks.joypadb) {
+				getJOYPADS()->sendJoystickButton(1, event.jbutton.button,
+					event.jbutton.state==SDL_PRESSED);
+			}
 			break;		
 	}
 }
 
+/*--- Video resize event ---*/
+
+static void process_resize_event(const SDL_Event &event)
+{
+	if (!host) {
+		return;
+	}
+
+	/* Use new size as fixed size */
+	if (bx_options.autozoom.fixedsize) {
+		bx_options.autozoom.width = event.resize.w;
+		bx_options.autozoom.height = event.resize.h;
+	}
+
+	host->video->resizeWindow(event.resize.w, event.resize.h);
+}
+
 ///////
 // main function for checking keyboard, mouse and joystick events
 // called from main.cpp every 20 ms
@@ -849,50 +967,33 @@ void check_event()
 	}
 
 	SDL_Event event;
-	int eventmask = SDL_EVENTMASK(SDL_KEYDOWN)
-					| SDL_EVENTMASK(SDL_KEYUP)
-					| SDL_EVENTMASK(SDL_MOUSEBUTTONDOWN)
-					| SDL_EVENTMASK(SDL_MOUSEBUTTONUP)
-					| SDL_EVENTMASK(SDL_MOUSEMOTION)
-					| SDL_EVENTMASK(SDL_JOYAXISMOTION)
-					| SDL_EVENTMASK(SDL_JOYHATMOTION)
-					| SDL_EVENTMASK(SDL_JOYBUTTONDOWN)
-					| SDL_EVENTMASK(SDL_JOYBUTTONUP)
-					| SDL_EVENTMASK(SDL_ACTIVEEVENT)
-#ifdef SDL_GUI
-					| SDL_EVENTMASK(GUI_RETURN_INFO)
-#endif
-					| SDL_EVENTMASK(SDL_QUIT);
 
-	SDL_PumpEvents();
-	while (SDL_PeepEvents(&event, 1, SDL_GETEVENT, eventmask)) {
-		int type = event.type;
-
-		if (type == SDL_KEYDOWN || type == SDL_KEYUP) {
-			process_keyboard_event(event);
-		}
-		else if (type == SDL_MOUSEBUTTONDOWN || type == SDL_MOUSEBUTTONUP
-				 || (type == SDL_MOUSEMOTION && grabbedMouse)) {
-			process_mouse_event(event);
-		}
-		else if (type == SDL_JOYAXISMOTION || type == SDL_JOYBUTTONDOWN
-			|| type == SDL_JOYBUTTONUP || type == SDL_JOYHATMOTION) {
-			process_joystick_event(event);
-		}
-		else if (type == SDL_ACTIVEEVENT) {
-			process_active_event(event);
-		}
-#ifdef SDL_GUI
-		else if (type == GUI_RETURN_INFO) {
-			int status = event.user.code;
-			if (status == STATUS_SHUTDOWN)
+	while (SDL_PollEvent(&event)) {
+		switch(event.type) {
+			case SDL_KEYDOWN:
+			case SDL_KEYUP:
+				process_keyboard_event(event);
+				break;
+			case SDL_MOUSEBUTTONDOWN:
+			case SDL_MOUSEBUTTONUP:
+			case SDL_MOUSEMOTION:
+				process_mouse_event(event);
+				break;
+			case SDL_JOYBUTTONDOWN:
+			case SDL_JOYBUTTONUP:
+			case SDL_JOYHATMOTION:
+			case SDL_JOYAXISMOTION:
+				process_joystick_event(event);
+				break;
+			case SDL_ACTIVEEVENT:
+				process_active_event(event);
+				break;
+			case SDL_VIDEORESIZE:
+				process_resize_event(event);
+				break;
+			case SDL_QUIT:
 				pendingQuit = true;
-			else if (status == STATUS_REBOOT)
-				RestartAll();
-		}
-#endif /* SDL_GUI */
-		else if (type == SDL_QUIT) {
-			pendingQuit = true;
+				break;
 		}
 	}
 
