@@ -1,7 +1,7 @@
 /*
  * hostfs.cpp - HostFS routines
  *
- * Copyright (c) 2001-2005 STanda of ARAnyM development team (see AUTHORS)
+ * Copyright (c) 2001-2008 STanda of ARAnyM development team (see AUTHORS)
  *
  * This file is part of the ARAnyM project which builds a new and powerful
  * TOS/FreeMiNT compatible virtual machine running on almost any hardware.
@@ -61,6 +61,107 @@
 
 #include "../../atari/hostfs/hostfs_nfapi.h"	/* XFS_xx and DEV_xx enum */
 
+// TODO FIXME the following POSIX emulation for MINGW32/WinAPI should be moved elsewhere...
+#ifdef __MINGW32__
+#define S_IXOTH	0
+#define S_IWOTH	0
+#define S_IROTH	0
+#define S_IXGRP	0
+#define S_IWGRP 0
+#define S_IRGRP 0
+#define S_ISVTX 0
+#define S_ISGID 0
+#define S_ISUID 0
+#define S_ISLNK(a) 0
+#define S_IFLNK 0
+#define O_NONBLOCK 0
+#define O_NOCTTY 0
+#define S_IRWXG 0
+#define S_IRWXO 0
+#define _PC_LINK_MAX 2
+
+static inline int readlink(const char *path, char *buf, size_t bufsiz)
+	{ errno = ENOSYS; return -1; }
+static inline int symlink(const char *oldpath, const char *newpath)
+	{ errno = ENOSYS; return -1; }
+static inline long pathconf(const char *file_name, int name)
+	{ errno = ENOSYS; return -1; }
+static inline int truncate(const char *file_name, off_t name)
+	{ errno = ENOSYS; return -1; } // FIXME truncate has to be there somewhere
+static inline int lstat(const char *file_name, struct stat *buf)
+	{ return stat(file_name, buf); }
+
+struct statfs
+{
+  long f_type;                  /* type of filesystem (see below) */
+  long f_bsize;                 /* optimal transfer block size */
+  long f_blocks;                /* total data blocks in file system */
+  long f_bfree;                 /* free blocks in fs */
+  long f_bavail;                /* free blocks avail to non-superuser */
+  long f_files;                 /* total file nodes in file system */
+  long f_ffree;                 /* free file nodes in fs */
+  long f_fsid;                  /* file system id */
+  long f_namelen;               /* maximum length of filenames */
+  long f_spare[6];              /* spare for later */
+};
+
+/* linux-compatible values for fs type */
+#define MSDOS_SUPER_MAGIC     0x4d44
+#define NTFS_SUPER_MAGIC      0x5346544E
+
+/**
+ * @author Prof. A Olowofoyeku (The African Chief)
+ * @author Frank Heckenbach
+ * @see http://gd.tuwien.ac.at/gnu/mingw/os-hacks.h
+ */
+int statfs(const char *path, struct statfs *buf)
+{
+  char tmp[MAX_PATH], resolved_path[MAX_PATH];
+  int retval = 0;
+
+  errno = 0;
+
+  // FIXME add realpath (the strcpy is just a hack!)
+  // realpath(path, resolved_path);
+  strcpy(resolved_path, path);
+
+  long sectors_per_cluster, bytes_per_sector;
+  if(!GetDiskFreeSpaceA(resolved_path, (DWORD *)&sectors_per_cluster,
+                        (DWORD *)&bytes_per_sector, (DWORD *)&buf->f_bavail,
+                        (DWORD *)&buf->f_blocks))
+  {
+    errno = ENOENT;
+    retval = -1;
+  }
+  else {
+    buf->f_bsize = sectors_per_cluster * bytes_per_sector;
+    buf->f_files = buf->f_blocks;
+    buf->f_ffree = buf->f_bavail;
+    buf->f_bfree = buf->f_bavail;
+  }
+
+  /* get the FS volume information */
+  if(strspn(":", resolved_path) > 0)
+    resolved_path[3] = '\0';    /* we want only the root */
+  if(GetVolumeInformation
+     (resolved_path, NULL, 0, (DWORD *)&buf->f_fsid, (DWORD *)&buf->f_namelen, NULL, tmp,
+      MAX_PATH))
+  {
+    if(strcasecmp("NTFS", tmp) == 0)
+    {
+      buf->f_type = NTFS_SUPER_MAGIC;
+    }
+    else {
+      buf->f_type = MSDOS_SUPER_MAGIC;
+    }
+  }
+  else {
+    errno = ENOENT;
+    retval = -1;
+  }
+  return retval;
+}
+#endif // __MINGW32__
 
 // please remember to leave this define _after_ the reqired system headers!!!
 // some systems does define this to some important value for them....
@@ -298,7 +399,10 @@ int32 HostFs::dispatch(uint32 fncode)
 
 		case XFS_READLABEL:
 			D(bug("%s", "fs_readlabel"));
-			ret = TOS_EINVFN;
+			fetchXFSC( &fc, getParameter(0) );
+			ret = xfs_readlabel( &fc, 
+								 (memptr)getParameter(1) /* buff */,
+								 getParameter(2) /* len */ );
 			break;
 
 		case XFS_SYMLINK:
@@ -700,7 +804,7 @@ int16 HostFs::flagsHost2Mint(int flags)
 	int16 res = 0; /* default read only */
 
 	/* exclusive */
-	if (!(flags & O_WRONLY|O_RDWR))
+	if (!(flags & (O_WRONLY|O_RDWR)))
 		res = 0;
 	if (flags & O_WRONLY)
 		res = 1; /* write only/ kludge to avoid files being created */
@@ -764,7 +868,7 @@ void HostFs::transformFileName( char* dest, const char* source )
 	}
 
 	// Get file name (strip off file extension)
-	char *dot = strrchr( source, '.' );
+	char *dot = (char *)strrchr( source, '.' );
 	// ignore leading dot
 	if (dot == source)
 		dot = NULL;
@@ -1057,6 +1161,56 @@ int32 HostFs::xfs_dfree( XfsCookie *dir, uint32 diskinfop )
 }
 
 
+//	fake Dreadlabel, extract the last folder name of the host path being root out the mount on 68k side
+int32 HostFs::xfs_readlabel(XfsCookie * dir, memptr buff, int16 len) {
+	// dir->drv->hostRoot contains the full path of the folder mounted to a drive
+	// to omitt very long label names, we stick to the last folder name
+	size_t	hostrootlen;
+	if (
+			dir && dir->drv && dir->drv->hostRoot
+		&&	(hostrootlen = strlen(dir->drv->hostRoot)) > 0
+	) {
+		//	it seems there is a host root path to be used
+		char* startchar = dir->drv->hostRoot;	//	position on start of name
+		char* poschar = &startchar[hostrootlen - 1];	//	position on last character
+		if (poschar > startchar && *poschar == *DIRSEPARATOR) {
+			//	ignore an ending slash
+			--poschar;
+		}
+		if (poschar > startchar && *poschar == ':') {
+			//	ignore an ending ":" from dos drive letters
+			--poschar;
+		}
+		char* endchar = poschar;	//	remember this position as the end of hostRoot to copy
+		//	search backwards for bounding slash
+		while (poschar > startchar && *poschar != *DIRSEPARATOR)
+			--poschar;
+		if (*poschar == *DIRSEPARATOR)
+			//	move to character behind that slash
+			++poschar;
+			
+		if (poschar <= endchar) {
+			//	there are some characters inbetween to copy
+			hostrootlen = endchar - poschar + 2;
+			if (len < 0 || hostrootlen > size_t(len)) {
+				return(TOS_ENAMETOOLONG);
+			} else {
+				Host2AtariSafeStrncpy(buff, poschar, hostrootlen);
+				return(TOS_E_OK);
+			}
+		}
+	}
+	if (len > 0) {
+		//	there is no label name to extract
+		//	fall back to a default label
+		Host2AtariSafeStrncpy(buff, "HOSTFS", len);
+		return TOS_E_OK;
+	} else {
+		return(TOS_ENAMETOOLONG);
+	}
+}
+
+
 int32 HostFs::xfs_mkdir( XfsCookie *dir, memptr name, uint16 mode )
 {
 	char fname[MAXPATHNAMELEN];
@@ -1499,7 +1653,7 @@ char *HostFs::host_readlink(const char *pathname, char *target, int len )
 		const char *slash = &pathname[strlen(pathname)];
 		while ( --slash >= pathname &&
 				*slash != '/' &&
-				*slash != '\\' );
+				*slash != '\\' ) ;
 
 		// get the path part (in front of the slash)
 		// and prepend it to the link taget (if fits)
@@ -1517,7 +1671,7 @@ char *HostFs::host_readlink(const char *pathname, char *target, int len )
 		slash = &target[strlen(target)];
 		while ( --slash >= target &&
 				*slash != '/' &&
-				*slash != '\\' );
+				*slash != '\\' ) ;
 		slash++;
 
 		if ( slash > target ) {
@@ -1703,9 +1857,13 @@ int32 HostFs::xfs_getxattr( XfsCookie *fc, memptr xattrp )
 	/* UWORD uid	   */  WriteInt16( xattrp + 12, statBuf.st_uid );	 // FIXME: this is Linux's one
 	/* UWORD gid	   */  WriteInt16( xattrp + 14, statBuf.st_gid );	 // FIXME: this is Linux's one
 	/* LONG	 size	   */  WriteInt32( xattrp + 16, statBuf.st_size );
+#ifdef __MINGW32__
+	/* LONG	 blksize   */  WriteInt32( xattrp + 20, 512 ); // FIXME: I just made up the number
+#else
 	/* LONG	 blksize   */  WriteInt32( xattrp + 20, statBuf.st_blksize );
-#if defined(OS_beos)
-	/* LONG	 nblocks   */  WriteInt32( xattrp + 24, 0 );
+#endif
+#if defined(OS_beos) || defined (__MINGW32__)
+	/* LONG	 nblocks   */  WriteInt32( xattrp + 24, 0 ); // FIXME: should be possible to find out for MINGW32
 #else
 	/* LONG	 nblocks   */  WriteInt32( xattrp + 24, statBuf.st_blocks );
 #endif
@@ -1766,7 +1924,7 @@ int32 HostFs::xfs_stat64( XfsCookie *fc, memptr statp )
 
 	/* LLONG hi size   */  WriteInt32( statp + 72, ( statBuf.st_size >> 32 ) & 0xffffffffUL );
 	/* LLONG lo size   */  WriteInt32( statp + 76,   statBuf.st_size & 0xffffffffUL );
-#if defined(OS_beos)
+#if defined(OS_beos) || defined(__MINGW32__)
 	/* LLONG hi blocks */  WriteInt32( statp + 80,   0 );
 	/* LLONG lo blocks */  WriteInt32( statp + 84,   0 );
 	/* ULONG    blksize*/  WriteInt32( statp + 88,   0 );
@@ -1970,7 +2128,7 @@ int32 HostFs::xfs_lookup( XfsCookie *dir, memptr name, XfsCookie *fc )
 
 	XfsFsFile *newFsFile;
 
-	if ( !fname || !*fname || (*fname == '.' && !fname[1]) ) {
+	if ( !*fname || (*fname == '.' && !fname[1]) ) {
 		newFsFile = dir->index;
 		newFsFile->refCount++;
 	} else if ( *fname == '.' && fname[1] == '.' && !fname[2] ) {
@@ -2325,7 +2483,7 @@ int32 HostFs::xfs_native_init( int16 devnum, memptr mountpoint, memptr hostroot,
 	drv->fsDevDrv = filesys_devdrv;
 	drv->mountPoint = strdup( fmountPoint );
 
-	int dnum = -1;
+	int16 dnum = -1;
 	if ( strlen( fmountPoint ) == 2 && fmountPoint[1] == ':' ) {
 		// The mountPoint is of a "X:" format: (BetaDOS mapping)
 		dnum = tolower(fmountPoint[0])-'a';
