@@ -1,7 +1,7 @@
 /*
  * parameters.cpp - parameter init/load/save code
  *
- * Copyright (c) 2001-2007 ARAnyM developer team (see AUTHORS)
+ * Copyright (c) 2001-2008 ARAnyM developer team (see AUTHORS)
  *
  * Authors:
  *  MJ		Milan Jurik
@@ -32,11 +32,16 @@
 #include "tools.h"		// for safe_strncpy()
 #include "host.h"
 #include "host_filesys.h"
+#include "cfgopts.h"
 
 #define DEBUG 0
 #include "debug.h"
 
-# include <cstdlib>
+#include <cstdlib>
+
+#ifdef OS_darwin
+#  include <CoreFoundation/CoreFoundation.h>
+#endif
 
 #ifndef USE_JIT
 # define USE_JIT 0
@@ -104,6 +109,7 @@ static struct option const long_options[] =
 #endif
   {"swap-ide", no_argument, 0, 'S'},
   {"emutos", no_argument, 0, 'e'},
+  {"locale", required_argument, 0, 'k'},
 #ifdef ENABLE_LILO
   {"lilo", no_argument, 0, 'l'},
 #endif
@@ -123,6 +129,7 @@ bool startupGUI = false;
 
 bool boot_emutos = false;
 bool boot_lilo = false;
+bool halt_on_reboot = false;
 bool ide_swap = false;
 uint32 FastRAMSize;
 
@@ -145,100 +152,6 @@ static bx_atadevice_options_t *diskc = &bx_options.atadevice[0][0];
 static bx_atadevice_options_t *diskd = &bx_options.atadevice[0][1];
 
 
-void expand_path(char *buf, size_t buf_size)
-{
-	char *path = buf;
-	if ( !strlen(path) )
-		return;
-
-	char prefix[2048];
-	size_t prefixLen = 0;
-
-	if ( path[0] == '~' && (path[1] == '/' || path[1] == '\\') ) {
-		// replace with the home folder path
-		Host::getHomeFolder(prefix, sizeof(prefix));
-		strcat(prefix, DIRSEPARATOR);
-		prefixLen = strlen( prefix );
-		path+=2;
-	} else if (path[0] == '*' && (path[1] == '/' || path[1] == '\\') ) {
-		Host::getDataFolder(prefix, sizeof(prefix));
-		strcat(prefix, DIRSEPARATOR);
-		prefixLen = strlen( prefix );
-		path+=2;
-	} else if ( path[0] != '/' && path[0] != '\\' && path[1] != ':' ) {
-		// path relative to config file
-		strcpy(prefix, config_file);
-		// find the last dirseparator
-		const char *slash = &prefix[strlen(prefix)];
-		while ( --slash >= prefix &&
-			*slash != '/' &&
-			*slash != '\\' );
-		// get the path part (in front of the slash)
-		prefixLen = slash - prefix + 1;
-	}
-
-	if ( prefixLen > 0 ) {
-		if ( buf_size >= prefixLen + strlen(path) ) {
-			memmove( buf + prefixLen, path, strlen(path)+1 );	
-			memmove( buf, prefix, prefixLen );
-		} else {
-			panicbug("Error - config entry size is insufficient");
-		}
-	}
-}
-
-void compress_path(char *path)
-{
-	if ( !strlen(path) )
-		return;
-
-	char prefix[2048];
-	size_t prefixLen = 0;
-	char *replacement = NULL;
-
-	// path relative to config file
-	strcpy(prefix, config_file);
-	// find the last dirseparator
-	const char *slash = &prefix[strlen(prefix)];
-	while ( --slash >= prefix &&
-		*slash != '/' &&
-		*slash != '\\' );
-	// get the path part (in front of the slash)
-	prefixLen = slash - prefix + 1;
-
-	if (prefixLen && strncmp(path, prefix, prefixLen) == 0) {
-		replacement = "";
-		D(bug("%s matches %.*s", path, prefixLen, prefix));
-	} 
-	else 
-	{
-		Host::getDataFolder(prefix, sizeof(prefix));
-		prefixLen = strlen(prefix);
-		if (prefixLen && strncmp(path, prefix, prefixLen) == 0) {
-			replacement = "*";
-			D(bug("%s matches %.*s", path, prefixLen, prefix));
-		} 
-		else 
-		{
-			/* Check if home prefix matches */
-			Host::getHomeFolder(prefix, sizeof(prefix));
-			prefixLen = strlen(prefix);
-			if (prefixLen && strncmp(path, prefix, prefixLen) == 0) {
-				replacement = "~";
-				D(bug("%s matches %.*s", path, prefixLen, prefix));
-			}
-		}
-	}
-
-	if (replacement) {
-		int len1 = strlen(replacement);
-		int len2 = strlen(path+prefixLen);
-		memmove(path+len1, path+prefixLen, len2+1);
-		memcpy(path, replacement, len1);
-	}
-}
-
-
 // configuration file 
 /*************************************************************************/
 struct Config_Tag global_conf[]={
@@ -284,20 +197,10 @@ void postload_global()
 #endif
 	if (!isalpha(bx_options.bootdrive))
 		bx_options.bootdrive = 0;
-
-	expand_path(bx_options.floppy.path, sizeof(bx_options.floppy.path));
-	expand_path(bx_options.tos_path, sizeof(bx_options.tos_path));
-	expand_path(bx_options.emutos_path, sizeof(bx_options.emutos_path));
-	expand_path(bx_options.bootstrap_path, sizeof(bx_options.bootstrap_path));
 }
 
 void presave_global()
 {
-	compress_path(bx_options.floppy.path);
-	compress_path(bx_options.tos_path);
-	compress_path(bx_options.emutos_path);
-	compress_path(bx_options.bootstrap_path);
-
 	bx_options.fastram = FastRAMSize / 1024 / 1024;
 	if (bx_options.bootdrive == 0)
 		bx_options.bootdrive = ' ';
@@ -339,7 +242,7 @@ struct Config_Tag jit_conf[]={
 
 void preset_jit()
 {
-	bx_options.jit.jit = false;
+	bx_options.jit.jit = true;
 	bx_options.jit.jitfpu = true;
 	bx_options.jit.jitcachesize = 8192;
 	bx_options.jit.jitlazyflush = 1;
@@ -365,6 +268,7 @@ void preset_tos()
 {
   bx_options.tos.redirect_CON = false;
   bx_options.tos.cookie_mch = 0x00050000; // ARAnyM
+  bx_options.tos.cookie_akp = -1;
 }
 
 void postload_tos()
@@ -420,6 +324,9 @@ void preset_opengl()
 
 void postload_opengl()
 {
+#ifndef ENABLE_OPENGL
+  bx_options.opengl.enabled = false;
+#endif
 }
 
 void presave_opengl()
@@ -443,7 +350,7 @@ void presave_opengl()
 BX_DISK_CONFIG(diskc);
 BX_DISK_CONFIG(diskd);
 
-static void set_ide(unsigned int number, char *dev_path, int cylinders, int heads, int spt, int byteswap, bool readonly, const char *model_name)
+static void set_ide(unsigned int number, const char *dev_path, int cylinders, int heads, int spt, int byteswap, bool readonly, const char *model_name)
 {
   // Autodetect ???
   if (cylinders == -1) {
@@ -508,14 +415,10 @@ void preset_ide()
 
 void postload_ide()
 {
-	expand_path(diskc->path, sizeof(diskc->path));
-	expand_path(diskd->path, sizeof(diskd->path));
 }
 
 void presave_ide()
 {
-	compress_path(diskc->path);
-	compress_path(diskd->path);
 }
 
 /*************************************************************************/
@@ -559,16 +462,10 @@ void preset_disk()
 
 void postload_disk()
 {
-	for(int i=0; i<DISKS; i++) {
-		expand_path(bx_options.disks[i].path, sizeof(bx_options.disks[i].path));
-	}
 }
 
 void presave_disk()
 {
-	for(int i=0; i<DISKS; i++) {
-		compress_path(bx_options.disks[i].path);
-	}
 }
 
 /*************************************************************************/
@@ -617,8 +514,6 @@ void preset_arafs()
 void postload_arafs()
 {
 	for(int i=0; i < 'Z'-'A'+1; i++) {
-		expand_path(bx_options.aranymfs[i].configPath, sizeof(bx_options.aranymfs[i].configPath));
-		
 		safe_strncpy(bx_options.aranymfs[i].rootPath, bx_options.aranymfs[i].configPath, sizeof(bx_options.aranymfs[i].rootPath));
 		int len = strlen(bx_options.aranymfs[i].configPath);
 		bx_options.aranymfs[i].halfSensitive = true;
@@ -641,7 +536,6 @@ void presave_arafs()
 			// set the halfSensitive indicator
 			strcat( bx_options.aranymfs[i].configPath, ":" );
 		}
-		compress_path(bx_options.aranymfs[i].configPath);
 	}
 }
 
@@ -716,14 +610,10 @@ void preset_lilo()
 
 void postload_lilo()
 {
-	expand_path(bx_options.lilo.kernel, sizeof(bx_options.lilo.kernel));
-	expand_path(bx_options.lilo.ramdisk, sizeof(bx_options.lilo.ramdisk));
 }
 
 void presave_lilo()
 {
-	compress_path(bx_options.lilo.kernel);
-	compress_path(bx_options.lilo.ramdisk);
 }
 
 /*************************************************************************/
@@ -743,11 +633,9 @@ void preset_midi() {
 }
 
 void postload_midi() {
-	expand_path(bx_options.midi.file, sizeof(bx_options.midi.file));
 }
 
 void presave_midi() {
-	compress_path(bx_options.midi.file);
 }
 
 /*************************************************************************/
@@ -878,7 +766,11 @@ void preset_natfeat() {
   safe_strncpy(NATFEAT_CONF(vdi_driver), "soft", sizeof(NATFEAT_CONF(vdi_driver)));
 }
 
-void postload_natfeat() {
+void postload_natfeat()
+{
+#ifndef ENABLE_OPENGL
+  safe_strncpy(NATFEAT_CONF(vdi_driver), "soft", sizeof(NATFEAT_CONF(vdi_driver)));
+#endif
 }
 
 void presave_natfeat() {
@@ -1004,6 +896,60 @@ void presave_ikbd()
 {
 }
 
+/*************************************************************************/
+struct Config_Tag audio_conf[]={
+	{ "Frequency", Int_Tag, &bx_options.audio.freq, 0, 0},
+	{ "Channels", Int_Tag, &bx_options.audio.chans, 0, 0},
+	{ "Bits", Int_Tag, &bx_options.audio.bits, 0, 0},
+	{ "Samples", Int_Tag, &bx_options.audio.samples, 0, 0},
+	{ NULL , Error_Tag, NULL, 0, 0 }
+};
+
+void preset_audio() {
+  bx_options.audio.freq = 22050;
+  bx_options.audio.chans = 2;
+  bx_options.audio.bits = 16;
+  bx_options.audio.samples = 1024;
+}
+
+void postload_audio() {
+}
+
+void presave_audio() {
+}
+
+/*************************************************************************/
+struct Config_Tag joysticks_conf[]={
+	{ "Ikbd0", Int_Tag, &bx_options.joysticks.ikbd0, 0, 0},
+	{ "Ikbd1", Int_Tag, &bx_options.joysticks.ikbd1, 0, 0},
+	{ "JoypadA", Int_Tag, &bx_options.joysticks.joypada, 0, 0},
+	{ "JoypadAButtons", String_Tag, &bx_options.joysticks.joypada_mapping,
+		sizeof(bx_options.joysticks.joypada_mapping), 0},
+	{ "JoypadB", Int_Tag, &bx_options.joysticks.joypadb, 0, 0},
+	{ "JoypadBButtons", String_Tag, &bx_options.joysticks.joypadb_mapping,
+		sizeof(bx_options.joysticks.joypadb_mapping), 0},
+	{ NULL , Error_Tag, NULL, 0, 0 }
+};
+
+void preset_joysticks() {
+	bx_options.joysticks.ikbd0 = -1;	/* This one is wired to mouse */
+	bx_options.joysticks.ikbd1 = 0;
+	bx_options.joysticks.joypada = -1;
+	bx_options.joysticks.joypadb = -1;
+	safe_strncpy(bx_options.joysticks.joypada_mapping,
+		"0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16",
+		sizeof(bx_options.joysticks.joypada_mapping));
+	safe_strncpy(bx_options.joysticks.joypadb_mapping,
+		"0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16",
+		sizeof(bx_options.joysticks.joypadb_mapping));
+}
+
+void postload_joysticks() {
+}
+
+void presave_joysticks() {
+}
+
 /*************************************************************************/
 void usage (int status) {
   printf ("Usage: %s [OPTIONS]\n", program_name);
@@ -1020,6 +966,7 @@ Options:\n\
   -s, --save                 save configuration file\n\
   -S, --swap-ide             swap IDE drives\n\
   -P <X,Y> or <center>       set window position\n\
+  -k, --locale <XY>          set NVRAM keyboard layout and language\n\
   -h, --help                 display this help and exit\n\
   -V, --version              output version information and exit\n\
 ");
@@ -1031,6 +978,7 @@ Options:\n\
 #endif
 #ifdef ENABLE_LILO
   printf("  -l, --lilo                 boot a linux kernel\n");
+  printf("  -H, --halt                 linux kernel halts on reboot\n");
 #endif
 #ifndef FixedSizeFastRAM
   printf("  -F, --fastram SIZE         FastRAM size (in MB)\n");
@@ -1062,6 +1010,8 @@ void preset_cfg() {
   preset_parallel();
   preset_natfeat();
   preset_nfvdi();
+  preset_audio();
+  preset_joysticks();
 }
 
 void postload_cfg() {
@@ -1085,6 +1035,8 @@ void postload_cfg() {
   postload_parallel();
   postload_natfeat();
   postload_nfvdi();
+  postload_audio();
+  postload_joysticks();
 }
 
 void presave_cfg() {
@@ -1108,6 +1060,8 @@ void presave_cfg() {
   presave_parallel();
   presave_natfeat();
   presave_nfvdi();
+  presave_audio();
+  presave_joysticks();
 }
 
 void early_cmdline_check(int argc, char **argv) {
@@ -1153,6 +1107,7 @@ int process_cmdline(int argc, char **argv)
 							 "e"  /* boot emutos */
 #ifdef ENABLE_LILO
 							 "l"  /* boot lilo */
+							 "H"  /* halt on reboot */
 #endif
 #ifdef DEBUGGER
 							 "D"  /* debugger */
@@ -1176,6 +1131,7 @@ int process_cmdline(int argc, char **argv)
 #endif
 							 "P:" /* position of the window */
 							 "S"  /* swap IDE drives */
+							 "k:" /* keyboard layout */
 							 "h"  /* help */
 							 "V"  /* version */,
 							 long_options, (int *) 0)) != EOF) {
@@ -1209,6 +1165,10 @@ int process_cmdline(int argc, char **argv)
 			case 'l':
 				boot_lilo = true;
 				break;
+
+			case 'H':
+				halt_on_reboot = true;
+				break;
 #endif
 
 			case 'v':
@@ -1270,6 +1230,28 @@ int process_cmdline(int argc, char **argv)
 				break;
 #endif
 
+			case 'k':
+				{
+					char countries[][3] = {{"us"},{"de"},{"fr"},{"uk"},{"es"},{"it"},{"se"},{"ch"},
+							{"cd"},{"tr"},{"fi"},{"no"},{"dk"},{"sa"},{"nl"},{"cz"},{"hu"},{"sk"},{"gr"}};
+					bx_options.tos.cookie_akp = -1;
+					for(unsigned i=0; i<sizeof(countries)/sizeof(countries[0]); i++) {
+						if (strcasecmp(optarg, countries[i]) == 0) {
+							bx_options.tos.cookie_akp = i << 8 | i;
+							break;
+						}
+					}
+					if (bx_options.tos.cookie_akp == -1) {
+						fprintf(stderr, "Error '%s', use one of:", optarg);
+						for(unsigned i=0; i<sizeof(countries)/sizeof(countries[0]); i++) {
+							fprintf(stderr, " %s", countries[i]);
+						}
+						fprintf(stderr, "\n");
+						exit(0);
+					}
+				}
+				break;
+				
 			default:
 				usage (EXIT_FAILURE);
 		}
@@ -1319,24 +1301,12 @@ char *getDataFilename(const char *file, char *buffer, unsigned int bufsize)
 	return addFilename(buffer, file, bufsize);
 }
 
-static int process_config(FILE *f, const char *filename, struct Config_Tag *conf, char *title, bool verbose)
-{
-	int status = input_config(filename, conf, title);
-	if (status >= 0) {
-		if (verbose)
-			fprintf(f, "%s configuration: found %d valid directives.\n", title, status);
-	} else {
-		fprintf(f, "Error while reading/processing the '%s' config file.\n", filename);
-	}
-	return status;
-}
-
-static bool decode_ini_file(FILE *f, const char *rcfile)
+static bool decode_ini_file(const char *rcfile)
 {
 	// Does the config exist?
 	struct stat buf;
 	if (stat(rcfile, &buf) == -1) {
-		fprintf(f, "Config file '%s' not found.\nThe config file is created with default values. Edit it to suit your needs.\n", rcfile);
+		infoprint("Config file '%s' not found. It's been created with default values. Edit it to suit your needs.", rcfile);
 		saveConfigFile = true;
 #ifdef SDL_GUI
 		startupGUI = true;
@@ -1344,48 +1314,58 @@ static bool decode_ini_file(FILE *f, const char *rcfile)
 		return false;
 	}
 
-	fprintf(f, "Using config file: '%s'\n", rcfile);
+	infoprint("Using config file: '%s'", rcfile);
 
-	process_config(f, rcfile, global_conf, "[GLOBAL]", true);
-	process_config(f, rcfile, startup_conf, "[STARTUP]", true);
-	process_config(f, rcfile, ikbd_conf, "[IKBD]", true);
-	process_config(f, rcfile, hotkeys_conf, "[HOTKEYS]", true);
-	process_config(f, rcfile, jit_conf, "[JIT]", true);
-	process_config(f, rcfile, video_conf, "[VIDEO]", true);
-	process_config(f, rcfile, tos_conf, "[TOS]", true);
-	process_config(f, rcfile, ide_swap ? diskd_configs : diskc_configs, "[IDE0]", true);
-	process_config(f, rcfile, ide_swap ? diskc_configs : diskd_configs, "[IDE1]", true);
+	bool verbose = false;
 
-	process_config(f, rcfile, disk0_configs, "[PARTITION0]", true);
-	process_config(f, rcfile, disk1_configs, "[PARTITION1]", true);
-	process_config(f, rcfile, disk2_configs, "[PARTITION2]", true);
-	process_config(f, rcfile, disk3_configs, "[PARTITION3]", true);
-	process_config(f, rcfile, disk4_configs, "[PARTITION4]", true);
-	process_config(f, rcfile, disk5_configs, "[PARTITION5]", true);
-	process_config(f, rcfile, disk6_configs, "[PARTITION6]", true);
-	process_config(f, rcfile, disk7_configs, "[PARTITION7]", true);
+	char home_folder[1024];
+	char data_folder[1024];
+	Host::getHomeFolder(home_folder, sizeof(home_folder));
+	Host::getDataFolder(data_folder, sizeof(data_folder));
+	ConfigOptions cfgopts(rcfile, home_folder, data_folder);
 
-	process_config(f, rcfile, arafs_conf, "[HOSTFS]", true);
-	process_config(f, rcfile, opengl_conf, "[OPENGL]", true);
-	process_config(f, rcfile, eth0_conf, "[ETH0]", true);
-	process_config(f, rcfile, eth1_conf, "[ETH1]", true);
-	process_config(f, rcfile, eth2_conf, "[ETH2]", true);
-	process_config(f, rcfile, eth3_conf, "[ETH3]", true);
-	process_config(f, rcfile, lilo_conf, "[LILO]", true);
-	process_config(f, rcfile, midi_conf, "[MIDI]", true);
-	process_config(f, rcfile, nfcdroms_conf, "[CDROMS]", true);
-	process_config(f, rcfile, autozoom_conf, "[AUTOZOOM]", true);
-	process_config(f, rcfile, osmesa_conf, "[NFOSMESA]", true);
-	process_config(f, rcfile, parallel_conf, "[PARALLEL]", true);
-	process_config(f, rcfile, natfeat_conf, "[NATFEATS]", true);
-	process_config(f, rcfile, nfvdi_conf, "[NFVDI]", true);
+	cfgopts.process_config(global_conf, "[GLOBAL]", verbose);
+	cfgopts.process_config(startup_conf, "[STARTUP]", verbose);
+	cfgopts.process_config(ikbd_conf, "[IKBD]", verbose);
+	cfgopts.process_config(hotkeys_conf, "[HOTKEYS]", verbose);
+	cfgopts.process_config(jit_conf, "[JIT]", verbose);
+	cfgopts.process_config(video_conf, "[VIDEO]", verbose);
+	cfgopts.process_config(tos_conf, "[TOS]", verbose);
+	cfgopts.process_config(ide_swap ? diskd_configs : diskc_configs, "[IDE0]", verbose);
+	cfgopts.process_config(ide_swap ? diskc_configs : diskd_configs, "[IDE1]", verbose);
+
+	cfgopts.process_config(disk0_configs, "[PARTITION0]", verbose);
+	cfgopts.process_config(disk1_configs, "[PARTITION1]", verbose);
+	cfgopts.process_config(disk2_configs, "[PARTITION2]", verbose);
+	cfgopts.process_config(disk3_configs, "[PARTITION3]", verbose);
+	cfgopts.process_config(disk4_configs, "[PARTITION4]", verbose);
+	cfgopts.process_config(disk5_configs, "[PARTITION5]", verbose);
+	cfgopts.process_config(disk6_configs, "[PARTITION6]", verbose);
+	cfgopts.process_config(disk7_configs, "[PARTITION7]", verbose);
+
+	cfgopts.process_config(arafs_conf, "[HOSTFS]", verbose);
+	cfgopts.process_config(opengl_conf, "[OPENGL]", verbose);
+	cfgopts.process_config(eth0_conf, "[ETH0]", verbose);
+	cfgopts.process_config(eth1_conf, "[ETH1]", verbose);
+	cfgopts.process_config(eth2_conf, "[ETH2]", verbose);
+	cfgopts.process_config(eth3_conf, "[ETH3]", verbose);
+	cfgopts.process_config(lilo_conf, "[LILO]", verbose);
+	cfgopts.process_config(midi_conf, "[MIDI]", verbose);
+	cfgopts.process_config(nfcdroms_conf, "[CDROMS]", verbose);
+	cfgopts.process_config(autozoom_conf, "[AUTOZOOM]", verbose);
+	cfgopts.process_config(osmesa_conf, "[NFOSMESA]", verbose);
+	cfgopts.process_config(parallel_conf, "[PARALLEL]", verbose);
+	cfgopts.process_config(natfeat_conf, "[NATFEATS]", verbose);
+	cfgopts.process_config(nfvdi_conf, "[NFVDI]", verbose);
+	cfgopts.process_config(audio_conf, "[AUDIO]", verbose);
+	cfgopts.process_config(joysticks_conf, "[JOYSTICKS]", verbose);
 
 	return true;
 }
 
 bool loadSettings(const char *cfg_file) {
 	preset_cfg();
-	bool ret = decode_ini_file(stderr, cfg_file);
+	bool ret = decode_ini_file(cfg_file);
 	postload_cfg();
 	return ret;
 }
@@ -1394,81 +1374,78 @@ bool saveSettings(const char *fs)
 {
 	presave_cfg();
 
-	if (update_config(fs, global_conf, "[GLOBAL]") < 0) {
+	char home_folder[1024];
+	char data_folder[1024];
+	Host::getHomeFolder(home_folder, sizeof(home_folder));
+	Host::getDataFolder(data_folder, sizeof(data_folder));
+	ConfigOptions cfgopts(fs, home_folder, data_folder);
+
+	if (cfgopts.update_config(global_conf, "[GLOBAL]") < 0) {
 		fprintf(stderr, "Error while writing the '%s' config file.\n", fs);
 		return false;
 	}
-	update_config(fs, startup_conf, "[STARTUP]");
-	update_config(fs, ikbd_conf, "[IKBD]");
-	update_config(fs, hotkeys_conf, "[HOTKEYS]");
-	update_config(fs, jit_conf, "[JIT]");
-	update_config(fs, video_conf, "[VIDEO]");
-	update_config(fs, tos_conf, "[TOS]");
-	update_config(fs, ide_swap ? diskd_configs : diskc_configs, "[IDE0]");
-	update_config(fs, ide_swap ? diskc_configs : diskd_configs, "[IDE1]");
+	cfgopts.update_config(startup_conf, "[STARTUP]");
+	cfgopts.update_config(ikbd_conf, "[IKBD]");
+	cfgopts.update_config(hotkeys_conf, "[HOTKEYS]");
+	cfgopts.update_config(jit_conf, "[JIT]");
+	cfgopts.update_config(video_conf, "[VIDEO]");
+	cfgopts.update_config(tos_conf, "[TOS]");
+	cfgopts.update_config(ide_swap ? diskd_configs : diskc_configs, "[IDE0]");
+	cfgopts.update_config(ide_swap ? diskc_configs : diskd_configs, "[IDE1]");
 
-	update_config(fs, disk0_configs, "[PARTITION0]");
+	cfgopts.update_config(disk0_configs, "[PARTITION0]");
 	if (strlen((char *)disk1_configs->buf))
-		update_config(fs, disk1_configs, "[PARTITION1]");
+		cfgopts.update_config(disk1_configs, "[PARTITION1]");
 	if (strlen((char *)disk2_configs->buf))
-		update_config(fs, disk2_configs, "[PARTITION2]");
+		cfgopts.update_config(disk2_configs, "[PARTITION2]");
 	if (strlen((char *)disk3_configs->buf))
-		update_config(fs, disk3_configs, "[PARTITION3]");
+		cfgopts.update_config(disk3_configs, "[PARTITION3]");
 	if (strlen((char *)disk4_configs->buf))
-		update_config(fs, disk4_configs, "[PARTITION4]");
+		cfgopts.update_config(disk4_configs, "[PARTITION4]");
 	if (strlen((char *)disk5_configs->buf))
-		update_config(fs, disk5_configs, "[PARTITION5]");
+		cfgopts.update_config(disk5_configs, "[PARTITION5]");
 	if (strlen((char *)disk6_configs->buf))
-		update_config(fs, disk6_configs, "[PARTITION6]");
+		cfgopts.update_config(disk6_configs, "[PARTITION6]");
 	if (strlen((char *)disk7_configs->buf))
-		update_config(fs, disk7_configs, "[PARTITION7]");
+		cfgopts.update_config(disk7_configs, "[PARTITION7]");
 
-	update_config(fs, arafs_conf, "[HOSTFS]");
-	update_config(fs, opengl_conf, "[OPENGL]");
+	cfgopts.update_config(arafs_conf, "[HOSTFS]");
+	cfgopts.update_config(opengl_conf, "[OPENGL]");
 
-	update_config(fs, eth0_conf, "[ETH0]");
+	cfgopts.update_config(eth0_conf, "[ETH0]");
 	if (strlen((char *)eth1_conf->buf))
-		update_config(fs, eth1_conf, "[ETH1]");
+		cfgopts.update_config(eth1_conf, "[ETH1]");
 	if (strlen((char *)eth2_conf->buf))
-		update_config(fs, eth2_conf, "[ETH2]");
+		cfgopts.update_config(eth2_conf, "[ETH2]");
 	if (strlen((char *)eth3_conf->buf))
-		update_config(fs, eth3_conf, "[ETH3]");
+		cfgopts.update_config(eth3_conf, "[ETH3]");
 
-	update_config(fs, lilo_conf, "[LILO]");
-	update_config(fs, midi_conf, "[MIDI]");
-	update_config(fs, nfcdroms_conf, "[CDROMS]");
-	update_config(fs, autozoom_conf, "[AUTOZOOM]");
-	update_config(fs, osmesa_conf, "[NFOSMESA]");
-	update_config(fs, parallel_conf, "[PARALLEL]");
-	update_config(fs, natfeat_conf, "[NATFEATS]");
-	update_config(fs, nfvdi_conf, "[NFVDI]");
+	cfgopts.update_config(lilo_conf, "[LILO]");
+	cfgopts.update_config(midi_conf, "[MIDI]");
+	cfgopts.update_config(nfcdroms_conf, "[CDROMS]");
+	cfgopts.update_config(autozoom_conf, "[AUTOZOOM]");
+	cfgopts.update_config(osmesa_conf, "[NFOSMESA]");
+	cfgopts.update_config(parallel_conf, "[PARALLEL]");
+	cfgopts.update_config(natfeat_conf, "[NATFEATS]");
+	cfgopts.update_config(nfvdi_conf, "[NFVDI]");
+	cfgopts.update_config(audio_conf, "[AUDIO]");
+	cfgopts.update_config(joysticks_conf, "[JOYSTICKS]");
 
 	return true;
 }
 
 bool check_cfg()
 {
-#if REAL_ADDRESSING
-# if defined(CPU_i386) && defined(OS_linux)
-	if (bx_options.fastram > (128 - 16))
-	{
-		panicbug("Maximum Fast RAM size for real addressing on x86/Linux is 112 MB");
-		panicbug("If you need bigger Fast RAM, you must recompile ARAnyM");
-		panicbug("Use ./configure --enable-addressing=direct");
-		return false;
-	}
-# endif
-#endif
 	return true;
 }
 
-bool decode_switches(FILE *f, int argc, char **argv)
+bool decode_switches(int argc, char **argv)
 {
 	getConfFilename(ARANYMCONFIG, config_file, sizeof(config_file));
 
 	early_cmdline_check(argc, argv);
 	preset_cfg();
-	decode_ini_file(f, config_file);
+	decode_ini_file(config_file);
 	process_cmdline(argc, argv);
 	postload_cfg();
 
